#include "ytmpv.hpp"

db::video getVideoByYTID(std::string yt_id) {
    db::video output;

    // 1. Проверить есть ли видео в базе данных
    if (db::getVideoByYTID(yt_id, &output)) {
        // Видео в базе данных найдено
        std::cout << "Видео найдено в базе данных!" << std::endl;
        return output;
    }

    // Видео в базе данных нет. Получаем информацию о нём через ytapi
    ytapi::video_result found_video = ytapi::getVideoByYTID(yt_id);
    output.yt_id        = yt_id;
    output.title        = found_video.title;
    output.description  = found_video.description;
    output.published_at = found_video.published_at;

    // Ищем автора в базе данных
    db::author video_author;
    if (!db::getAuthorByYTID(found_video.author_yt_id, &video_author)) {
        // Автора в базе данных нет. Добавляем
        video_author.yt_id = found_video.author_yt_id;
        video_author.name = found_video.author_name;
        db::addAuthor(&video_author);
    }
    output.author_obj = video_author;

    // Добавляем видео в БД
    db::addVideo(&output);

    return output;
}

int main(int argc, char** argv) {
    // ==Инициализация==
    // --Язык--
    std::locale::global(std::locale(""));

    // --Аргументы приложения--
    Glib::ustring config_path;
    Glib::OptionEntry opt_config_path = Glib::OptionEntry();
    opt_config_path.set_long_name("config");
    opt_config_path.set_short_name('c');
    opt_config_path.set_flags(Glib::OptionEntry::Flags::NONE);
    opt_config_path.set_description("Path to JSON configuration file");
    opt_config_path.set_arg_description("<path to config>");

    Glib::OptionContext option_context = Glib::OptionContext(
        "- launcher for mpv to watch youtube videos"
    );
    Glib::OptionGroup main_group = Glib::OptionGroup(
        "Main option group",
        "main option description",
        "no help."
    );
    main_group.add_entry(opt_config_path, config_path);
    option_context.set_main_group(main_group);

    try {
        option_context.parse(argc, argv);
    } catch (Glib::OptionError* e) {
        std::cout << e->what();
        return 1;
    }

    // --Получение данных из файла настроек--
    // TODO
    std::string db_path = ":memory:";
    std::string api_key = "key";
    
    // --Общая инициализация--
    curl_global_init(CURL_GLOBAL_ALL);
    db::databaseInit(db_path);
    db::createTables();
    ytapi::youtubeInit(api_key);

    // ==Запуск приложения==
    auto app = Gtk::Application::create("com.github.aquadim.YTMPV");
    return app->make_window_and_run<components::MainWindow>(argc, argv);

    //~ db::video found;
    //~ std::vector<std::string> video_ids = ytapi::getVideoIDsByQuery(argv[3]);
    //~ for (auto id : video_ids) {
        //~ found = getVideoByYTID(id);

        //~ std::cout << found.title << ":" << found.author_obj.name << std::endl;
    //~ }

    //~ std::ifstream ifs("example.json");
    //~ std::string content(
        //~ (std::istreambuf_iterator<char>(ifs)),
        //~ (std::istreambuf_iterator<char>())
    //~ );

    //~ ytapi::getVideosFromJSON(content);
}